( cd usr/lib ; rm -rf libzrtpcpp.so )
( cd usr/lib ; ln -sf libzrtpcpp-0.9.so.0.0.1 libzrtpcpp.so )
( cd usr/lib ; rm -rf libzrtpcpp-0.9.so.0 )
( cd usr/lib ; ln -sf libzrtpcpp-0.9.so.0.0.1 libzrtpcpp-0.9.so.0 )
